//-----------------------------------------------------------------------------
// FILE:              JavaCommand.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include <sstream>
#include "JavaCommand.h"
#include "util/Trace.h"
#include "util/tstring.h"
#include "com_softtrac_jni_NativeCommand.h"

using namespace std;

namespace SoftTrac {

static Trace trace("JavaCommand", Trace::OFF);


//-----------------------------------------------------------------------------
// JavaCommand::command
//-----------------------------------------------------------------------------
JavaCommand* JavaCommand::command(istream& is)
{
   JavaMessage* message = JavaMessage::message(is);
   JavaCommand* command = dynamic_cast<JavaCommand*>(message);
   trace.debug("command", toString(command));

   return command;
}


//-----------------------------------------------------------------------------
// JavaCommand::execute
//-----------------------------------------------------------------------------
string JavaCommand::execute(istream& methodArgs)
{
   parseMethodArgs(methodArgs);
   realExecute();
   return formatReturnArgs();
}


//-----------------------------------------------------------------------------
// JavaCommand::JavaCommand
//-----------------------------------------------------------------------------
JavaCommand::JavaCommand(const std::string& nativeCommandName)
   :
      JavaMessage(nativeCommandName)
{
}


} // namespace SoftTrac


//-----------------------------------------------------------------------------
// Java_com_softtrac_jni_NativeCommand_execute
//-----------------------------------------------------------------------------
JNIEXPORT jstring JNICALL Java_com_softtrac_jni_NativeCommand_execute
  (JNIEnv * env, jobject, jstring args)
{
   using namespace SoftTrac;
   using namespace std;

   static const string FUNCTION = "Java_com_softtrac_jni_NativeCommand_execute";
   static Trace trace("");
   trace.enter(FUNCTION);

   JavaMessage::Sentry sentry(env);

   //---
   // Make copy of event and listener name passed in from java.
   //---
   jboolean owner;
   istringstream myArgs(env->GetStringUTFChars(args, &owner));

   //---
   // Map the command name to its corresponding c++ object.
   //---
   JavaCommand* command = JavaCommand::command(myArgs);

   //---
   // Run the command.
   //---
   string returnArgs = (command) ? command->execute(myArgs) 
                                 : string("null");

   //---
   // Now that we have a deep copy of the command name, 
   // we can release it on the java side.
   //---
   if (owner == JNI_TRUE)
   {
      env->ReleaseStringUTFChars(args, myArgs.str().c_str());
   }

   //---
   // Send the command return arguments (if any) back to java.
   //---
   trace.exit(FUNCTION);
   return env->NewStringUTF(returnArgs.c_str());
}
