//-----------------------------------------------------------------------------
// FILE:              JavaListener.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include <assert.h>
#include "JavaListener.h"
#include "util/Trace.h"
#include "util/tstring.h"

using namespace std;

namespace SoftTrac {

static Trace trace("JavaListener");


//-----------------------------------------------------------------------------
// JavaListener::~JavaListener
//-----------------------------------------------------------------------------
JavaListener::~JavaListener()
{
   clearEvent();
}


//-----------------------------------------------------------------------------
// JavaListener::listener
//-----------------------------------------------------------------------------
JavaListener* JavaListener::listener(const string& listenerName)
{
   JavaMessage* message = JavaMessage::message(listenerName);
   JavaListener* listener = dynamic_cast<JavaListener*>(message);
   trace.debug("listener", toString(listener));

   return listener;
}


//-----------------------------------------------------------------------------
// JavaListener::JavaListener
//-----------------------------------------------------------------------------
JavaListener::JavaListener(const std::string& nativeListenerName)
   :
      JavaMessage (nativeListenerName),
      theEvent    (0)
{
}


//-----------------------------------------------------------------------------
// JavaListener::forwardNotify
//-----------------------------------------------------------------------------
void JavaListener::forwardNotify() 
{
   static const string METHOD = "forwardNotify";
 
   assert(theJNIEnv);

   trace.enter(METHOD);

   jclass NativeEvent = theJNIEnv->GetObjectClass(theEvent);

   jmethodID notify = theJNIEnv->GetMethodID(NativeEvent, 
					                         "notify",
                                             "(Ljava/lang/String;)V");
   
   theJNIEnv->CallObjectMethod(
      theEvent, 
      notify, 
      theJNIEnv->NewStringUTF(formatReturnArgs().c_str())); 

   trace.exit(METHOD);
}


//-----------------------------------------------------------------------------
// JavaListener::setEvent
//-----------------------------------------------------------------------------
void JavaListener::setEvent(jobject event)
{
   const string METHOD = "setEvent";
   trace.enter(METHOD);
   assert(theJNIEnv);

   clearEvent();
   theEvent = theJNIEnv->NewGlobalRef(event);

   trace.exit(METHOD);
}


//-----------------------------------------------------------------------------
// JavaListener::clearEvent
//-----------------------------------------------------------------------------
void JavaListener::clearEvent()
{
   if (theJNIEnv && theEvent) 
   {
      theJNIEnv->DeleteGlobalRef(theEvent);
      theEvent = 0;
   }
}


} // namespace SoftTrac
