//-----------------------------------------------------------------------------
// FILE:              JavaMessage.h
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_JavaMessage_H
#define SoftTrac_JavaMessage_H

#include <iosfwd>
#include <map>
#include <string>
#include <sstream>
#include <vector>
#include <jni.h>

namespace SoftTrac {

//-----------------------------------------------------------------------------
// CLASS:  JavaMessage
//-----------------------------------------------------------------------------
class JavaMessage
{
public:
   virtual ~JavaMessage();

   static JavaMessage* message(const std::string&);
   static JavaMessage* message(std::istream&);

   class Sentry
   {
   public:
      Sentry(JNIEnv* env) { JavaMessage::theJNIEnv = env; }
     ~Sentry() { JavaMessage::theJNIEnv = 0; }
   };

protected:
   JavaMessage(const std::string& nativeClassName);

   void put(const std::string& keyword, bool value); 
   void put(const std::string& keyword, char value);
   void put(const std::string& keyword, int value);
   void put(const std::string& keyword, long value);
   void put(const std::string& keyword, float value);
   void put(const std::string& keyword, double value);

   void put(const std::string& keyword, const char*        value);
   void put(const std::string& keyword, const std::string& value);

   void put(const std::string& keyword, const std::vector<bool>& value);
   void put(const std::string& keyword, const std::vector<int>& value);
   void put(const std::string& keyword, const std::vector<long>& value);
   void put(const std::string& keyword, const std::vector<float>& value);
   void put(const std::string& keyword, const std::vector<double>& value);

   void put(void* reference);
   
   bool   getBool  (const std::string& keyword);
   char   getChar  (const std::string& keyword);
   int    getInt   (const std::string& keyword);
   long   getLong  (const std::string& keyword);
   float  getFloat (const std::string& keyword);
   double getDouble(const std::string& keyword);
   
   std::string getString(const std::string& keyword);

   std::vector<bool>   getBoolArray  (const std::string& keyword);
   std::vector<int>    getIntArray   (const std::string& keyword);
   std::vector<long>   getLongArray  (const std::string& keyword);
   std::vector<float>  getFloatArray (const std::string& keyword);
   std::vector<double> getDoubleArray(const std::string& keyword);
   
   void* getReference();

   std::string     theNativeClassName;
   static JNIEnv*  theJNIEnv;

private:
   typedef std::map<std::string,JavaMessage*,std::less<std::string> > JNI_Map;
   typedef std::map<std::string,std::string, std::less<std::string> > KV_Map;
      
   KV_Map theMethodArgs;
   KV_Map theReturnArgs;

   static JNI_Map* theMessages;

   friend class Sentry;

public:
   void        parseMethodArgs(std::istream&);
   std::string formatReturnArgs() const;
};


//-----------------------------------------------------------------------------
// writeContainer
//-----------------------------------------------------------------------------
template <class Container, class T>
std::string writeContainer(const Container& c, const T&)
{
   ostringstream oss;
   oss << c.size() << " [ ";
   for (Container::const_iterator i = c.begin(); i != c.end(); ++i)
   {
      oss << (T)(*i) << ' ';
   }
   oss << ']';

   return oss.str();
}


//-----------------------------------------------------------------------------
// readContainer
//-----------------------------------------------------------------------------
template <class Container, class T>
void readContainer(Container& c, const T&, std::string& value)
{
   int  length;
   T    tmp;
   char start, finish;
   istringstream iss(value);

   iss >> length >> start;
   if (length <= 0 || start != '[') throw std::invalid_argument("Bad container");

   for (int i = 0; i < length; ++i)
   {
      iss >> tmp;
      c.push_back((T)tmp);
   }

   iss >> finish;
   if (finish != ']') throw std::invalid_argument("Bad container");
}


} // namespace SoftTrac


#endif


