//-----------------------------------------------------------------------------
// FILE:              NativeEvent.java
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------

package com.softtrac.jni;

import com.softtrac.util.Trace;

/**
 * The NativeEvent class is ...
 *
 * @author Timothy R. Culp
 * @version %W% %G%
 */
public abstract class NativeEvent extends NativeMessage {

   public void addNativeListener(NativeListener listener)
      throws NativeEvent.Exception {
      //>
      final String METHOD = "addNativeListener";
      trace.enter(METHOD);
      theListener = listener;
      if (attach(this.nativeProxy(), listener.nativeProxy()).equals("null")) {
         throw new BadEvent(nativeProxy());         
      }
      trace.exit(METHOD);
   } //<
   
   public void removeNativeListener(NativeListener listener)
      throws NativeEvent.Exception {
      //>
      trace.enter("removeNativeListener");
      theListener = listener;
      if (detach(this.nativeProxy(), listener.nativeProxy()).equals("null")) {
         throw new BadEvent(nativeProxy());         
      }
      trace.exit("removeNativeListener");
      //<
   }

   public class BadEvent extends Exception {
      public BadEvent(String eventName) {
         //>
         super("BadEvent", eventName);
         //<
      }
   }
   protected NativeEvent(String remoteProxy) {
      //>
      super(remoteProxy);
      //<
   }
   
   protected void notify(String returnArgs)
      throws NativeEvent.Exception {
      //>
      final String METHOD="notify";
       
      trace.enter(METHOD);
      trace.log(METHOD, returnArgs);
      parseReturnArgs(returnArgs);
      theListener.update(this);
      trace.enter(METHOD);
      //<
   }
   
   private native String attach(String eventName, String listenerName);
   private native String detach(String eventName, String listenerName);
   
   private NativeListener theListener;
   
   private static Trace trace;
   static {
      //>
      trace = new Trace("jni.NativeEvent");
      // trace.setAll();
      //<
   }
}


   








