//-----------------------------------------------------------------------------
// FILE:              Battery.h
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_Battery_h
#define SoftTrac_Battery_h

#include "util/Event.h"

namespace SoftTrac {

class Observer;

//---------------------------------------------------------------------------
// CLASS: Battery
//---------------------------------------------------------------------------
class Battery 
{
public:
   Battery(int power=100);

   void drawPower(int amount);

   void setPower(int power) { theCurrentPower = theStartPower = power; } 
   int  getPower() const { return theCurrentPower; }

   void attachFullPowerEvent(Observer* o)   {theEvents.attach(o,FULL_POWER);  }
   void attachPowerChangeEvent(Observer* o) {theEvents.attach(o,POWER_CHANGE);}
   void attachLowPowerEvent(Observer* o)    {theEvents.attach(o,LOW_POWER);   }
   void attachNoPowerEvent(Observer* o)     {theEvents.attach(o,NO_POWER);    }

   void detachFullPowerEvent(Observer* o)   {theEvents.detach(o,FULL_POWER);  }
   void detachPowerChangeEvent(Observer* o) {theEvents.detach(o,POWER_CHANGE);}
   void detachLowPowerEvent(Observer* o)    {theEvents.detach(o,LOW_POWER);   }
   void detachNoPowerEvent(Observer* o)     {theEvents.detach(o,NO_POWER);    }

   enum 
   {
      FULL_POWER   = 0,
      POWER_CHANGE = 1,
      LOW_POWER    = 2,
      NO_POWER     = 3,
      NUM_EVENTS   = 4
   };

   typedef SubjectEvent<Battery> Event;

private:
   int   theStartPower;
   int   theCurrentPower;
   Event theEvents;
};

} // namespace SoftTrac

#endif
