//-----------------------------------------------------------------------------
// FILE:              BatteryImpl.h
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_BatteryImpl_h
#define SoftTrac_BatteryImpl_h

#include "Battery.h"
#include "jni/JavaCommand.h"
#include "jni/JavaEvent.h"
#include "jni/JavaListener.h"

class SoftTrac::Event;

namespace SoftTrac {
namespace BatteryImpl {


//-----------------------------------------------------------------------------
// CLASS: ConstructBattery 
//-----------------------------------------------------------------------------
class ConstructBattery : public JavaCommand
{
public:
   ConstructBattery(const std::string& cmd);
   virtual void realExecute();
};


//-----------------------------------------------------------------------------
// CLASS: DrawPowerCommand 
//-----------------------------------------------------------------------------
class DrawPowerCommand : public JavaCommand
{
public:
   DrawPowerCommand(const std::string& cmd);
   virtual void realExecute();
};


//-----------------------------------------------------------------------------
// CLASS: FullPowerEvent 
//-----------------------------------------------------------------------------
class FullPowerEvent : public JavaEvent
{
public:
   FullPowerEvent(const std::string& event);
   virtual void attach(JavaListener* listener);
   virtual void detach(JavaListener* listener);
};


//-----------------------------------------------------------------------------
// CLASS: PowerChangeEvent 
//-----------------------------------------------------------------------------
class PowerChangeEvent : public JavaEvent
{
public:
   PowerChangeEvent(const std::string& event);
   virtual void attach(JavaListener* listener);
   virtual void detach(JavaListener* listener);
};


//-----------------------------------------------------------------------------
// CLASS: LowPowerEvent 
//-----------------------------------------------------------------------------
class LowPowerEvent : public JavaEvent
{
public:
   LowPowerEvent(const std::string& event);
   virtual void attach(JavaListener* listener);
   virtual void detach(JavaListener* listener);
};


//-----------------------------------------------------------------------------
// CLASS: NoPowerEvent 
//-----------------------------------------------------------------------------
class NoPowerEvent : public JavaEvent
{
public:
   NoPowerEvent(const std::string& event);
   virtual void attach(JavaListener* listener);
   virtual void detach(JavaListener* listener);
};


//-----------------------------------------------------------------------------
// CLASS: FullPowerListener 
//-----------------------------------------------------------------------------
class FullPowerListener : public JavaListener
{
public:
   FullPowerListener(const std::string& listener);
   virtual void update(Event* event) throw(std::invalid_argument);
};


//-----------------------------------------------------------------------------
// CLASS: PowerChangeListener 
//-----------------------------------------------------------------------------
class PowerChangeListener : public JavaListener
{
public:
   PowerChangeListener(const std::string& listener);
   virtual void update(Event* event) throw(std::invalid_argument);
};


//-----------------------------------------------------------------------------
// CLASS: LowPowerListener 
//-----------------------------------------------------------------------------
class LowPowerListener : public JavaListener
{
public:
   LowPowerListener(const std::string& listener);
   virtual void update(Event* event) throw(std::invalid_argument);
};


//-----------------------------------------------------------------------------
// CLASS: NoPowerListener 
//-----------------------------------------------------------------------------
class NoPowerListener : public JavaListener
{
public:
   NoPowerListener(const std::string& listener);
   virtual void update(Event* event) throw(std::invalid_argument);
};


} // namespace BatteryImpl
} // namespace SoftTrac


#endif
