//-----------------------------------------------------------------------------
// FILE:              JBattery.java
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

package com.softtrac.jni.example;

import com.softtrac.jni.*;
import com.softtrac.util.Trace;
import com.softtrac.util.Assertion;

public class JBattery
{
   public static final int FullPower = 0;
   public static final int PowerChange = 1;
   public static final int LowPower = 2;
   public static final int NoPower = 3;
   public static final int NumEvents = 4;

   public JBattery(int startPower) throws NativeMessage.Exception {
      //>
      theStartPower = startPower;

      //---
      // Create a native battery
      //---
      NativeCommand command = new ConstructBattery();
      command.execute();

      //---
      // Create a list of possible events to attach to
      //---
      theEvents = new NativeEvent[NumEvents];
      theEvents[FullPower] = new FullPowerEvent();
      theEvents[PowerChange] = new PowerChangeEvent();
      theEvents[LowPower] = new LowPowerEvent();
      theEvents[NoPower] = new NoPowerEvent();
   }
   //<

   //---
   // your code goes here
   //---
   public void drawPower(int amount) throws NativeMessage.Exception {
      //>
      NativeCommand command = new DrawPowerCommand(amount);
      command.execute();
   }
   //<
   
   public void addNativeListener(NativeListener listener)
      throws NativeMessage.Exception {
      //>
      //---
      // Get event number from listener
      //---
      int eventNo = listener.eventNo();
      new Assertion(eventNo >= 0 && eventNo < NumEvents); 

      //---
      // Get the event and add the listener
      //---
      NativeEvent event = theEvents[eventNo];
      event.addNativeListener(listener);
   }
   //<
   
   //---
   // Native commands
   //---
   class ConstructBattery extends NativeCommand {
      ConstructBattery() {
         //>
         super("JBattery.ConstructBattery");
      }
      //<
      protected String nativeProxy() {
         //>
         return "BatteryImpl::ConstructBattery";
      }
      //<
      protected void preExecute() {
         //>
         //---
         // Put any command arguments
         //---
         put("start_power", theStartPower);
      }
      //<
      protected void postExecute() {
         //>
         //---
         // Get any return arguments
         //---
         theBattery = getReference();
         new Assertion(theBattery != 0);
      }
      //<
   }

   class DrawPowerCommand extends NativeCommand {
      DrawPowerCommand(int amount) {
         //>
         super("JBattery.DrawPowerCommand");
         theAmount = amount;
      }
      //<
      protected String nativeProxy() {
         //>
         return "BatteryImpl::DrawPowerCommand";
      }
      //<
      protected void preExecute() {
         //>
         //---
         // Put any command arguments
         //---
         put("amount", theAmount);
      }
      //<
      protected void postExecute() {
         //>
         //---
         // Get any return arguments
         //---
         // int value1 = getInt("keyword1");
         // double value2 = getDouble("keyword2");
      }
      //<
      private int theAmount;
   }

   //---
   // Native events
   //---
   class FullPowerEvent extends NativeEvent {
      FullPowerEvent() {
         //>
         super("JBattery.FullPowerEvent");
      }
      //<
      protected String nativeProxy() {
         //>
         return "BatteryImpl::FullPowerEvent";
      }
      //<
   }

   class PowerChangeEvent extends NativeEvent {
      PowerChangeEvent() {
         //>
         super("JBattery.PowerChangeEvent");
      }
      //<
      protected String nativeProxy() {
         //>
         return "BatteryImpl::PowerChangeEvent";
      }
      //<
   }

   class LowPowerEvent extends NativeEvent {
      LowPowerEvent() {
         //>
         super("JBattery.LowPowerEvent");
      }
      //<
      protected String nativeProxy() {
         //>
         return "BatteryImpl::LowPowerEvent";
      }
      //<
   }

   class NoPowerEvent extends NativeEvent {
      NoPowerEvent() {
         //>
         super("JBattery.NoPowerEvent");
      }
      //<
      protected String nativeProxy() {
         //>
         return "BatteryImpl::NoPowerEvent";
      }
      //<
   }

   private NativeEvent[] theEvents;
   private long          theBattery;
   private int           theStartPower;

   static {
      System.loadLibrary("Debug/jni");
   }
}
