//-----------------------------------------------------------------------------
// FILE:              JBatteryMonitor.java
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

package com.softtrac.jni.example;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import com.softtrac.jni.*;
import com.softtrac.util.*;


/**
 * JBatteryMonitor
 */
public class JBatteryMonitor extends JFrame
{
   public JBatteryMonitor(int startPower) throws Exception {
      //>
      super("Battery Monitor");

      //---
      // Create the battery and attach some listeners
      //---
      theStartPower = theCurrentPower = startPower;
      theBattery = new JBattery(startPower);
      theBattery.addNativeListener(new PowerChangeListener());
      theBattery.addNativeListener(new LowPowerListener());
      theBattery.addNativeListener(new NoPowerListener());

      //---
      // Create the buttons
      //---
      theOnButton = new JButton("On");
      theOnButton.setActionCommand("on");
      theOnButton.addActionListener(new OnButtonListener());
      
      theOffButton = new JButton("Off");
      theOffButton.setActionCommand("off");
      theOffButton.addActionListener(new OffButtonListener());
      theOffButton.setEnabled(false);

      //---
      // Create the progress bar
      //---
      theProgressBar = new JProgressBar(0, theStartPower);
      theProgressBar.setValue(theStartPower);
      theProgressBar.setStringPainted(true);

      //---
      // Create the output area
      //---
      theOutput = new JTextArea(5, 20);
      theOutput.setMargin(new Insets(5,5,5,5));
      theOutput.setEditable(false);

      //---
      // Add components to some panels
      //---
      JPanel panel = new JPanel();
      panel.add(theOnButton);
      panel.add(theOffButton);
      panel.add(theProgressBar);
      
      JPanel contentPane = new JPanel();
      contentPane.setLayout(new BorderLayout());
      contentPane.add(panel, BorderLayout.NORTH);
      contentPane.add(new JScrollPane(theOutput), BorderLayout.CENTER);
      contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
      setContentPane(contentPane);
   }
   //<

   void    on() {
      //>
      theMonitorIsOff = false;
      DrawPowerThread drawPower = new DrawPowerThread();
      drawPower.start();
   }
   //<
   void    off() { theMonitorIsOff = true; }
   boolean isDead() { return theCurrentPower == 0; }
   String  getStatus() {
      //>
      return "Drawn " + theCurrentPower + "V out of " + theStartPower + "V.";
   }
   //<

   class OnButtonListener implements ActionListener {
      public void actionPerformed(ActionEvent evt) {
         //>
         theOnButton.setEnabled(false);
         theOffButton.setEnabled(true);
         on();
      }
      //<
   }
   
   class OffButtonListener implements ActionListener {
      public void actionPerformed(ActionEvent evt) {
         //>
         theOffButton.setEnabled(false);
         theOnButton.setEnabled(true);
         off();
      }
      //<
   }
   
   class PowerChangeListener implements NativeListener {
      public void   update(NativeEvent event) {
         //>
         theCurrentPower = event.getInt("power");
         String status = getStatus();
         System.out.println(status);
         theProgressBar.setValue(theCurrentPower);
         theOutput.append(status + newline);
         theOutput.setCaretPosition(theOutput.getDocument().getLength());
         if (isDead()) theProgressBar.setValue(theProgressBar.getMinimum());
      }
      //<
      public String nativeProxy() {
         //>
         return "BatteryImpl::PowerChangeListener";
      }
      //<
      public int    eventNo() {
         //>
         return JBattery.PowerChange;
      }
      //<
   }

   class LowPowerListener implements NativeListener {
      public void   update(NativeEvent event) {
         //>         
         theProgressBar.setForeground(Color.red);
         System.out.println("Warning: Battery power level is low...");
      }
      //<
      public String nativeProxy() {
         //>
         return "BatteryImpl::LowPowerListener";
      }
      //<
      public int    eventNo() {
         //>
         return JBattery.LowPower;
      }
      //<
   }

   class NoPowerListener implements NativeListener {
      public void   update(NativeEvent event) {
         //>
         theOffButton.setEnabled(false);
         theOnButton.setEnabled(false);
         off();
         System.out.println("Error: The battery is dead.");
      }
      //<
      public String nativeProxy() {
         //>
         return "BatteryImpl::NoPowerListener";
      }
      //<
      public int    eventNo() {
         //>
         return JBattery.NoPower;
      }
      //<
   }

   class DrawPower {
      DrawPower() {
         //>
         while (!theMonitorIsOff) {
            try {
               theBattery.drawPower((int)(Math.random()*10));
               Thread.sleep(JBatteryMonitor.ONE_SECOND); 
            } catch (Exception e) { System.err.println(e.getMessage()); }
         }
      }
      //<
   }

   class DrawPowerThread extends SwingWorker {
      public Object construct() { return new DrawPower(); }
   }

   public static void main(String[] args) throws Exception {
      //>
      int startPower = 100;
      if (args.length > 0) {
         Integer argPower = new Integer(args[0]);
         startPower = argPower.intValue();
      }
      JFrame frame = new JBatteryMonitor(startPower);
      
      frame.addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent e) {
            System.exit(0);
         }
      });
      
      frame.pack();
      frame.setVisible(true);
   }
   //<

   public final static int ONE_SECOND = 1000;
   
   private int            theStartPower;
   private int            theCurrentPower;
   private JBattery       theBattery;
   private boolean        theBatteryIsDead;
   private boolean        theMonitorIsOff;
   private String         theStatusMessage;
   private JProgressBar   theProgressBar;
   private JButton        theOnButton;
   private JButton        theOffButton;
   private JTextArea      theOutput;
   private String         newline = "\n";   
}







