package com.softtrac.nomess;

import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import com.softtrac.jni.*;
import com.softtrac.util.Trace;
import com.softtrac.util.Time;


public class BoardController
{
   public BoardController(Frame parent, int boardSize)
      throws NativeMessage.Exception {
      //>
      final String METHOD = "Board";
      trace.enter(METHOD);

      theParent = parent;
      theMainPanel = new Panel(new BorderLayout());
      theParent.add(theMainPanel);

      Font font = new Font("Courier", Font.BOLD, 18);
      
      //---
      // Create the Board and attach some listeners
      //---
      theBoard = new Board(boardSize);
      theBoard.addNativeListener(new ChangeListener());
      theBoard.addNativeListener(new CleanupListener());

      //---
      // Create the score panel
      //---
      theScorePanel = new Panel(new BorderLayout());
      theScorePanel.setBackground(Color.lightGray);
      theScoreLabel = new Label("Score 0");
      theScorePanel.add(theScoreLabel, BorderLayout.EAST);
      theScorePanel.setFont(font);
      theLookAheadLabel = new Label();
      theLookAheadLabel.setFont(font);
      theScorePanel.add(theLookAheadLabel, BorderLayout.WEST);
      
      //---
      // Create the board panel
      //---
      theBoardPanel = new Panel(new GridLayout(boardSize, boardSize));
      theButtons = new Button[boardSize * boardSize];
      int i = 0;
      for (int row = 0; row < boardSize; ++row)
      {
         for (int col = 0; col < boardSize; ++col)
         {
	    theButtons[i] = new Button(new Square(row, col));
	    theButtons[i].setFont(font);
            theButtons[i].addActionListener(new ButtonListener());
            theBoardPanel.add(theButtons[i]);
            ++i;
         }
      }

      theMainPanel.add(theScorePanel, BorderLayout.NORTH);
      theMainPanel.add(theBoardPanel, BorderLayout.CENTER);
      
      trace.exit(METHOD);
   } //<

   Board board() { return theBoard; }

   void selectStart(Button button) {
      //>
      try {
         Square square = button.square();
         if (board().isOccupied(square))
         {
            theStartButton = button;
            theStartButton.setBackground(Color.gray); 
            theStartIsSelected = true;
         }
      } catch (Exception e) { System.err.println(e.getMessage()); }
   } //<
   void selectFinish(Button button) {
      //>
      try {
         Square square = button.square();
         if (!board().isOccupied(square))
         {
            theFinishButton = button;
            theStartButton.setBackground(Color.lightGray); 
            theFinishButton.setBackground(Color.gray);
            Time.pause(.25);
            theFinishButton.setBackground(Color.lightGray); 
            theFinishIsSelected = true;
            
            boolean valid = board().move(theStartButton.square(),
                                         theFinishButton.square());

            if (!valid) switchSides();  
         }
         else if (theStartButton.square().equals(square))
         {
            button.setBackground(Color.lightGray); 
            theStartIsSelected = false;
         }
         else
         {
            theStartButton.setBackground(Color.lightGray); 
            button.setBackground(Color.gray);
            theStartButton = button;
         }
      } catch (Exception e) { System.err.println(e.getMessage()); }
   } //<
   void switchSides() {
      //>
      theStartIsSelected = false;
      theFinishIsSelected = false;
   } //<
   void cleanup(int row, String tokens) {
      //>
      for (int col = 0; col < theBoard.getSize(); ++col) 
      {
         char token = tokens.charAt(col);
         if (token == ' ') 
         {
            Button button = getButton(row, col);
            button.setLabel(" ");
         }
      }
   } //<

   Button getButton(int row, int col) {
      //>
      int index = row*9 + col;
      return theButtons[index];
   } //<

   void   setLookAheadLabel(String text) {
      //>
      theLookAheadLabel.setText(text);
   } //<
   
   class ChangeListener implements NativeListener {
      public void   update(NativeEvent event) {
         //>
         //---
         // Get any return arguments
         //---
         int  row   = event.getInt("row");
         int  col   = event.getInt("col");
         char token = event.getChar("token");
         
         Button button = getButton(row, col);
	 button.setLabel(""+token);
         if (token != ' ') Time.pause(.25);
      } //<
      public String nativeProxy() {
         //>
         return "BoardServer::ChangeListener";
      } //<
      public int    eventNo() {
         //>
         return Board.Change;
      } //<
   }

   class CleanupListener implements NativeListener {
      public void   update(NativeEvent event) {
         //>
         //---
         // Get any return arguments
         //---
         theScore = event.getInt("score");
         theScoreLabel.setText("Score "+theScore);
                  
         for (int row = 0; row < theBoard.getSize(); ++row)
         {
            String tokens = event.getString("row"+row);
            cleanup(row, tokens);
         }
      } //<
      public String nativeProxy() {
         //>
         return "BoardServer::CleanupListener";
      } //<
      public int    eventNo() {
         //>
         return Board.Cleanup;
      } //<
   }
   
   class Button extends java.awt.Button {
      Button(Square square) {
         //>
         super(" ");
         theSquare = square;
      } //<
      Square square() { return theSquare; }
      private Square theSquare;
   }
   
   class ButtonListener implements ActionListener {
      public void actionPerformed(ActionEvent ae) {
         //>
         Button button = (Button)ae.getSource();
         
         if      (!theStartIsSelected)  selectStart(button);
         else if (!theFinishIsSelected) selectFinish(button);
      } //<
   }
   
   private Frame    theParent;
   private Panel    theMainPanel;
   private Board    theBoard;
   private Panel    theScorePanel;
   private Label    theScoreLabel;
   private int      theScore;
   private Label    theLookAheadLabel;
   
   private Panel    theBoardPanel;
   private Button[] theButtons;
   private boolean  theStartIsSelected;
   private Button   theStartButton;
   private boolean  theFinishIsSelected;
   private Button   theFinishButton;
   
   private static Trace trace;
   static {
      //>
      trace = new Trace("nomess.BoardController");
      // trace.setAll();
   } //<
}








