//----------------------------------------------------------------- 
// FILE: BoardIterator.cpp
//----------------------------------------------------------------- 
   
#include "tconfig.h"
#include <sstream>
#include <stdexcept>
#include "BoardIterator.h"
#include "Board.h"

namespace SoftTrac {


//----------------------------------------------------------------- 
// Row::Row
//----------------------------------------------------------------- 
Row::Row(Board* b, unsigned int n) 
   : 
      theBoard (b),
      theRowNo (n)
{
   if (n >= b->numRows())
   {
      std::ostringstream what;
      what << "Bad row: " << n;
      throw std::out_of_range(what.str().c_str());
   }
}


//----------------------------------------------------------------- 
// Row::end
//----------------------------------------------------------------- 
Row::iterator Row::end()   
{ 
   return RowIterator(theBoard, 
                      Square(theRowNo, theBoard->numRows())); 
}


//----------------------------------------------------------------- 
// Col::Col
//----------------------------------------------------------------- 
Col::Col(Board* b, unsigned int n) 
   : 
      theBoard (b),
      theColNo (n)
{
   if (n >= b->numCols())
   {
      std::ostringstream what;
      what << "Bad column: " << n;
      throw std::out_of_range(what.str().c_str());
   }
}


//---------------------------------------------------------------- 
// Col::end
//----------------------------------------------------------------- 
Col::iterator Col::end()   
{ 
   return ColIterator(theBoard, 
                      Square(theBoard->numCols(), theColNo)); 
}


//----------------------------------------------------------------- 
// BackDiag::BackDiag
//----------------------------------------------------------------- 
BackDiag::BackDiag(Board* b, unsigned int n) 
   : 
      theBoard(b)
{
   const int SIZE  = b->size();
   const int SIZE0 = b->size()-1;

   if (n < SIZE)
   {
      theBegin = Square(0, SIZE0-n);
      theEnd   = Square(n+1, SIZE);
   }
   else if (n >= SIZE && n < b->numDiags())
   {
      theBegin = Square(n-SIZE0, 0);
      theEnd   = Square(SIZE, 2*SIZE-1-n);
   }
   else
   {
      std::ostringstream what;
      what << "Bad diagonal: " << n;
      throw std::out_of_range(what.str().c_str());
   }
}


//----------------------------------------------------------------- 
// FwdDiag::FwdDiag
//----------------------------------------------------------------- 
FwdDiag::FwdDiag(Board* b, unsigned int n) 
   : 
      theBoard(b)
{
   const int SIZE  = b->size();
   const int SIZE0 = b->size()-1;

   if (n < SIZE)
   {
      theBegin = Square(0, n);
      theEnd   = Square(n+1, -1);
   }
   else
   {
      theBegin = Square(n-SIZE0, SIZE0);
      theEnd   = Square(SIZE, n-SIZE);
   }
}


//----------------------------------------------------------------- 
// BoardIterator::operator*
//----------------------------------------------------------------- 
const Token* BoardIterator::operator*() const
{ 
   return theBoard->getToken(theSquare); 
}


//----------------------------------------------------------------- 
// BoardIterator::operator*
//----------------------------------------------------------------- 
Token* BoardIterator::operator*()
{ 
   return theBoard->getToken(theSquare); 
}


//----------------------------------------------------------------- 
// RowIterator::operator++
//----------------------------------------------------------------- 
RowIterator RowIterator::operator++(int) 
{
   RowIterator oldValue = *this;
   ++(*this);
   return oldValue;
}


//----------------------------------------------------------------- 
// ColIterator::operator++
//----------------------------------------------------------------- 
ColIterator ColIterator::operator++(int) 
{
   ColIterator oldValue = *this;
   ++(*this);
   return oldValue;
}


//----------------------------------------------------------------- 
// BackDiagIterator::operator++
//----------------------------------------------------------------- 
BackDiagIterator BackDiagIterator::operator++(int) 
{
   BackDiagIterator oldValue = *this;
   ++(*this);
   return oldValue;
}


//----------------------------------------------------------------- 
// FwdDiagIterator::operator++
//----------------------------------------------------------------- 
FwdDiagIterator FwdDiagIterator::operator++(int) 
{
   FwdDiagIterator oldValue = *this;
   ++(*this);
   return oldValue;
}


} // namespace SoftTrac
