#ifndef SoftTrac_BoardServer_h
#define SoftTrac_BoardServer_h

#include "Board.h"
#include "jni/JavaCommand.h"
#include "jni/JavaEvent.h"
#include "jni/JavaListener.h"

namespace SoftTrac {

class Event;

//-----------------------------------------------------------------------------
// CLASS: BoardServer
//-----------------------------------------------------------------------------
class BoardServer
{
public:
   static Board* subject(int size);
   static Board* subject();

   //---
   // Remote commands
   //---
   class ConstructCommand : public JavaCommand
   {
   public:
      ConstructCommand(const std::string& cmd);
      virtual void realExecute();
   };

   class IsOccupiedCommand : public JavaCommand
   {
   public:
      IsOccupiedCommand(const std::string& cmd);
      virtual void realExecute();
   };

   class MoveCommand : public JavaCommand
   {
   public:
      MoveCommand(const std::string& cmd);
      virtual void realExecute();
   };

   //---
   // Remote events
   //---
   class ChangeEvent : public JavaEvent
   {
   public:
      ChangeEvent(const std::string& event);
      virtual void attach(JavaListener* listener);
      virtual void detach(JavaListener* listener);
   };

   class MoveCompleteEvent : public JavaEvent
   {
   public:
      MoveCompleteEvent(const std::string& event);
      virtual void attach(JavaListener* listener);
      virtual void detach(JavaListener* listener);
   };

   class CleanupEvent : public JavaEvent
   {
   public:
      CleanupEvent(const std::string& event);
      virtual void attach(JavaListener* listener);
      virtual void detach(JavaListener* listener);
   };

   //---
   // Remote listeners
   //---
   class ChangeListener : public JavaListener
   {
   public:
      ChangeListener(const std::string& listener);
      virtual void update(Event* event) throw(std::invalid_argument);
   };

   class MoveCompleteListener : public JavaListener
   {
   public:
      MoveCompleteListener(const std::string& listener);
      virtual void update(Event* event) throw(std::invalid_argument);
   };

   class CleanupListener : public JavaListener
   {
   public:
      CleanupListener(const std::string& listener);
      virtual void update(Event* event) throw(std::invalid_argument);
   };

private:
  static Board* theBoard;
};

} // namespace SoftTrac

#endif
