//----------------------------------------------------------------- 
// FILE: ComputerPlayer.h
//----------------------------------------------------------------- 
#ifndef ComputerPlayer_h
#define ComputerPlayer_h

#include <vector>
#include "Player.h"
#include "util/Event.h"
#include "util/Observer.h"

namespace SoftTrac {

class Token;

//----------------------------------------------------------------- 
// CLASS: ComputerPlayer
//----------------------------------------------------------------- 
class ComputerPlayer : public Player
{
public:
   ComputerPlayer(Board* board=0);

   virtual void setBoard(Board* board);
   virtual void nextMove();

   void increaseDifficulty() { ++theSkillLevel; }

   const std::vector<const Token*> lookAheadQueue() const 
      { return theLookAheadQueue; }

   void attachNewLookAheadEvent(Observer* o) {theEvents.attach(o,NEW_LOOKAHEAD);}
   void detachNewLookAheadEvent(Observer* o) {theEvents.detach(o,NEW_LOOKAHEAD);}

   enum 
   {
      NEW_LOOKAHEAD = 0,
      NUM_EVENTS    = 1
   };

private:
   class LevelChangeObserver : public Observer
   {
   public:
      LevelChangeObserver(ComputerPlayer* cp) : theComputerPlayer(cp) {}
      virtual void update(Event*) throw (std::invalid_argument);
      virtual std::string name() const;
   private:
      ComputerPlayer* theComputerPlayer;
   };

   enum { TOKENS_TO_PLAY = 3 };

   typedef SubjectEvent<ComputerPlayer> Events;

   void placeTokens(unsigned int tokensToPlay);

   LevelChangeObserver theObserver;
  
   std::vector<const Token*> theLookAheadQueue;

   Events theEvents;
   int    theSkillLevel;
};


} // namespace SoftTrac

#endif
