package com.softtrac.nomess;

import com.softtrac.jni.*;
import com.softtrac.util.Assertion;

public class ComputerPlayer
{
   public static final int NewLookAhead = 0;
   public static final int NumEvents = 1;

   public ComputerPlayer() {
      //>
      //---
      // Create the list of events
      //---
      theEvents = new NativeEvent[NumEvents];
      theEvents[NewLookAhead] = new NewLookAheadEvent();
   } //<

   public void addNativeListener(NativeListener listener)
      throws NativeMessage.Exception {
      //>
      //---
      // Get event number from listener
      //---
      int eventNo = listener.eventNo();
      new Assertion(eventNo >= 0 && eventNo < NumEvents);

      //---
      // Get the event and add the listener
      //---
      NativeEvent event = theEvents[eventNo];
      event.addNativeListener(listener);
   } //<

   public void setBoard(Board board) throws NativeMessage.Exception {
      //>
      //---
      // Create a native ComputerPlayer
      //---
      NativeCommand command = new ConstructCommand(board);
      command.execute();
   } //<

   void move() throws NativeMessage.Exception {
      //>
      NativeCommand command = new NextMoveCommand();
      command.execute();
   } //<
   
   //---
   // Native commands
   //---
   class ConstructCommand extends NativeCommand {
      ConstructCommand(Board board) {
         //>
         super("ComputerPlayer.ConstructCommand");
         theBoard = board;
      } //<
      protected void preExecute() {
         //>
         //---
         // Put any command arguments
         //---
         put(theBoard.reference());
      } //<
      protected void postExecute() {
         //>
         //---
         // Get any return arguments
         //---
         theComputerPlayer = getReference();
      } //<
      protected String nativeProxy() {
         //>
         return "ComputerPlayerServer::ConstructCommand";
      }
      //<
      private Board theBoard;
   }

   class NextMoveCommand extends NativeCommand {
      NextMoveCommand() {
         //>
         super("ComputerPlayer.NextMoveCommand");
      } //<
      protected String nativeProxy() {
         //>
         return "ComputerPlayerServer::NextMoveCommand";
      }
      //<
   }

   //---
   // Native events
   //---
   class NewLookAheadEvent extends NativeEvent {
      NewLookAheadEvent() {
         //>
         super("ComputerPlayer.NewLookAheadEvent");
      } //<
      protected String nativeProxy() {
         //>
         return "ComputerPlayerServer::NewLookAheadEvent";
      }
      //<
   }

   private long theComputerPlayer;
   private NativeEvent[] theEvents;

   static {
      //>
      System.loadLibrary("Debug/NoMess");
   } //<
}





