package com.softtrac.nomess;

import java.awt.event.*;
import com.softtrac.jni.*;
import com.softtrac.util.Trace;

public class ComputerPlayerController
{
   public ComputerPlayerController(BoardController boardCtrl)
      throws NativeMessage.Exception {
      //>
      final String METHOD = "ComputerPlayerController";
      trace.enter(METHOD);

      theBoardCtrl = boardCtrl;
      
      //---
      // Add listener to the board
      //---
      Board board = boardCtrl.board();
      board.addNativeListener(new MoveCompleteListener());
      
      //---
      // Create the ComputerPlayer and attach some listeners
      //---
      theComputerPlayer = new ComputerPlayer();
      theComputerPlayer.addNativeListener(new NewLookAheadListener());
      theComputerPlayer.setBoard(board);
      
      trace.exit(METHOD);
   } //<

   class MoveCompleteListener implements NativeListener {
      public void   update(NativeEvent event) {
         //>
         try {
            theComputerPlayer.move();
            theBoardCtrl.switchSides();
         }
         catch (NativeMessage.Exception e) {
            System.err.println(e.getMessage());
         }
      } //<
      public String nativeProxy() {
         //>
         return "BoardServer::MoveCompleteListener";
      } //<
      public int    eventNo() {
         //>
         return Board.MoveComplete;
      } //<
   }

   class NewLookAheadListener implements NativeListener {
      public void   update(NativeEvent event) {
         //>
         //---
         // Get any return arguments
         //---
         String lookAhead = event.getString("lookahead");
         theBoardCtrl.setLookAheadLabel(lookAhead);
      } //<
      public String nativeProxy() {
         //>
         return "ComputerPlayerServer::NewLookAheadListener";
      } //<
      public int    eventNo() {
         //>
         return ComputerPlayer.NewLookAhead;
      } //<
   }
   
   private ComputerPlayer  theComputerPlayer;
   private BoardController theBoardCtrl;
   
   private static Trace trace;
   static {
      //>
      trace = new Trace("nomess.ComputerPlayerController");
      // trace.setAll();
      //<
   }
}


