//-----------------------------------------------------------------------------
// FILE:              Event.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include "Event.h"
#include "util/Observer.h"
#include "util/Trace.h"
#include "util/tstring.h"

using namespace std;

namespace SoftTrac {


static Trace trace("Event");


//-----------------------------------------------------------------------------
// Event::Event
//-----------------------------------------------------------------------------
Event::Event(const std::string& name, unsigned int numEvents) 
   : 
      theName   (name), 
      theEvents (numEvents) 
{
}


//-----------------------------------------------------------------------------
// Event::~Event
//-----------------------------------------------------------------------------
Event::~Event()
{
}


//-----------------------------------------------------------------------------
// Event::attach
//-----------------------------------------------------------------------------
void Event::attach(Observer* observer, unsigned int eventNo)
{
   Observers& observers = theEvents[eventNo];
   observers.push_back(observer);
   trace.debug("attach", observer->name());
}


//-----------------------------------------------------------------------------
// Event::detach
//-----------------------------------------------------------------------------
void Event::detach(Observer* observer, unsigned int eventNo)
{
   static const string METHOD = "detach";

   Observers& observers = theEvents[eventNo];
   observers.remove(observer);

   trace.debug(METHOD, observer->name());
   trace.debug(METHOD, "observers:\t"+toString(observers.size()));
}


//-----------------------------------------------------------------------------
// Event::notify
//-----------------------------------------------------------------------------
void Event::notify(unsigned int eventNo)
{
   static const string METHOD = "notify";

   Observers& observers = theEvents[eventNo];
   trace.debug(METHOD, "observers:\t"+toString(observers.size()));

   Observers::iterator observer = observers.begin();
   Observers::iterator end      = observers.end();
   for (; observer != end; ++observer) 
   {
      (*observer)->update(this);
      trace.debug(METHOD, (*observer)->name()+" notified");
   }
}


} // namespace SoftTrac

