//-----------------------------------------------------------------------------
// FILE:              Event.h
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_Event_h
#define SoftTrac_Event_h

#include <list>
#include <string>
#include <vector>

namespace SoftTrac {

class Observer;

//-----------------------------------------------------------------------------
// CLASS: Event
//-----------------------------------------------------------------------------
class Event
{
public:
   Event(const std::string& name, unsigned int numEvents);
   virtual ~Event();

   void attach(Observer*, unsigned int eventNo);
   void detach(Observer*, unsigned int eventNo);

   void notify(unsigned int eventNo);

   const std::string& name() const { return theName; }

private:
   typedef std::list<Observer*>   Observers;
   typedef std::vector<Observers> Events;
   
   std::string theName;
   Events      theEvents;
};

//-----------------------------------------------------------------------------
// CLASS: SubjectEvent
//-----------------------------------------------------------------------------
template <class Subject>
class SubjectEvent : public Event
{
public:
   SubjectEvent(Subject*           subject, 
                const std::string& subjectName,
                unsigned int       numEvents)
      : 
         Event(subjectName, numEvents), theSubject(subject) 
   {}

   Subject* subject() { return theSubject; }

   static Subject* cast(Event* event)
   {
      SubjectEvent<Subject>* s = dynamic_cast<SubjectEvent<Subject>*>(event);
      return (s) ? s->theSubject : 0;
   }

private:
   Subject* theSubject;
};

} // namespace SoftTrac

#endif

