//-----------------------------------------------------------------------------
// FILE:              Memento.h
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#ifndef SoftTrac_Memento_H
#define SoftTrac_Memento_H

#include <iosfwd>
#include "util/exceptions/Exception.h"

namespace SoftTrac {


//-----------------------------------------------------------------------------
// CLASS: Memento
//-----------------------------------------------------------------------------
template <class Originator, class State>
class Memento
{
public:
   virtual ~Memento(){}

   friend ostream& operator<<(ostream&, const Memento&);

protected:
   Memento(const Originator*, const State&);

   const State& state(const Originator*) const;

   const Originator* theOriginator;
   State             theState;
};


//-----------------------------------------------------------------------------
// operator<<
//-----------------------------------------------------------------------------
template <class Originator, class State>
std::ostream& operator<<(std::ostream&                         os,
                         const Memento<Originator,State>& m)
{
   return os << m.theState;

}


//-----------------------------------------------------------------------------
// Memento<Originator,State>::Memento
//-----------------------------------------------------------------------------
template <class Originator, class State>
inline Memento<Originator,State>::Memento(const Originator* o,
				                              const State&      s)
   :
      theOriginator (o),
      theState      (s)
{
}


//-----------------------------------------------------------------------------
// Memento<Originator,State>::state
//-----------------------------------------------------------------------------
template <class Originator, class State>
const State&
Memento<Originator,State>::state(const Originator* o) const
{
   if (o != theOriginator)
   {
      throw InvalidArgument("Originator does not match state",
			    "Memento<Originator,State>::state()",
			    __LINE__,
                            __FILE__);
   }

   return theState;
}


} // namespace SoftTrac


#endif
