//-----------------------------------------------------------------------------
// FILE:              Subject.h
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_Subject_h
#define SoftTrac_Subject_h

#include <list>
#include <vector>

namespace SoftTrac {

class Observer;

//-----------------------------------------------------------------------------
// CLASS: Subject
//-----------------------------------------------------------------------------
class Subject
{
public:
   virtual ~Subject();

   void attach(Observer*, unsigned int eventNo);
   void detach(Observer*, unsigned int eventNo);

   void notify(unsigned int eventNo);

protected:
   Subject(unsigned int numEvents) : theEvents(numEvents) {}

private:
   typedef std::list<Observer*>   Observers;
   typedef std::vector<Observers> Events;
   
   Events theEvents;
};

//-----------------------------------------------------------------------------
// CLASS: RealSubject
//-----------------------------------------------------------------------------
template <class T>
class RealSubject : public Subject
{
public:
   RealSubject(T* realSubject, unsigned int numEvents)
    : Subject(numEvents), theRealSubject(realSubject) {}

   static T* downCast(Subject* subject)
   {
      RealSubject<T>* realSubject = dynamic_cast<RealSubject<T>*>(subject);
      return (realSubject) ? realSubject->theRealSubject : 0;
   }

private:
   T* theRealSubject;
};

} // namespace SoftTrac

#endif

