//-----------------------------------------------------------------------------
// FILE:              TestDriver.cpp
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include "./TestDriver.h"
#include "./TestHarness.h"

namespace SoftTrac {


//-----------------------------------------------------------------------------
// TestDriver::execute
//-----------------------------------------------------------------------------
void TestDriver::execute(TestHarness* harness)
{
   theTestHarness = harness;
   
   preExecute();
   execute();
   postExecute();
}
   

//-----------------------------------------------------------------------------
// TestDriver::compare
//-----------------------------------------------------------------------------
void TestDriver::compare(bool               expr,
                         const std::string& module, 
                         int                line, 
                         const std::string& file) const 
   throw (AssertionError)
{
   if (!expr)
   {
      std::ostringstream reason;
      reason << "false expression";
      throw AssertionError("AssertionError", reason.str(), 
                           module, line, file);
   }
}


//-----------------------------------------------------------------------------
// TestDriver::compareNot
//-----------------------------------------------------------------------------
void TestDriver::compareNot(bool               expr,
                            const std::string& module, 
                            int                line, 
                            const std::string& file) const 
   throw (AssertionError)
{
   if (expr)
   {
      std::ostringstream reason;
      reason << "true expression";
      throw AssertionError("AssertionError", reason.str(), 
                           module, line, file);
   }
}


//-----------------------------------------------------------------------------
// TestDriver::TestDriver
//-----------------------------------------------------------------------------
TestDriver::TestDriver(const std::string& driverName)
   :
      theDriverName  (driverName),
      theTestHarness (0)
{
   TestHarness::registerDriver(this);
}


//-----------------------------------------------------------------------------
// TestDriver::logInfo
//-----------------------------------------------------------------------------
std::ostream& TestDriver::logInfo(uint               line,
                                   const std::string& file)
{
   return logStream(theTestHarness->infoStream(), line, file);
}


//-----------------------------------------------------------------------------
// TestDriver::logResult
//-----------------------------------------------------------------------------
std::ostream& TestDriver::logResult(uint               line,
                                     const std::string& file)
{
   return logStream(theTestHarness->resultsStream(), line, file);
}
 

//-----------------------------------------------------------------------------
// TestDriver::logError
//-----------------------------------------------------------------------------
std::ostream& TestDriver::logError(uint               line,
                                    const std::string& file)
{
   return logStream(theTestHarness->errorStream(), line, file);
}


//-----------------------------------------------------------------------------
// TestDriver::testing
//-----------------------------------------------------------------------------
void TestDriver::testing(const std::string& module, 
                          unsigned int       line, 
                          const std::string& file)
{
   logInfo(line, file) << "Testing module " << module << "..." << std::endl;
}


//-----------------------------------------------------------------------------
// TestDriver::logStream
//-----------------------------------------------------------------------------
std::ostream& TestDriver::logStream(std::ostream&      os, 
                                     uint               line,
                                     const std::string& file)
{
   return Exception::format(os, line, file);
   return os;
}


} // namespace SoftTrac
