//-----------------------------------------------------------------------------
// FILE:              TestDriver.h
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_TestDriver_H
#define SoftTrac_TestDriver_H

#include <assert.h>
#include <iostream>
#include <sstream>
#include <string>
#include <errno.h>
#include <stdlib.h>
#include "util/exceptions/commonExceptions.h"

namespace SoftTrac {

class TestHarness;

//-----------------------------------------------------------------------------
// CLASS: TestDriver
//-----------------------------------------------------------------------------
class TestDriver
{
public:
   void execute(TestHarness*);
 
   const std::string& name() const { return theDriverName; }

   typedef NamedException<DomainError> AssertionError;

   void compare(bool               expr,
                const std::string& module, 
                int                line=0, 
                const std::string& file="") const throw (AssertionError);

   template <class T>
   void compare(const T&           t1, 
                const T&           t2,
                const std::string& module, 
                int                line=0, 
                const std::string& file="") const throw (AssertionError)
   {
      if ( !(t1==t2) )
      {
         std::ostringstream reason;
         reason << t1 << "!=" << t2;
         throw AssertionError("AssertionError", reason.str(), 
                              module, line, file);
      }
   }


   void compareNot(bool               expr,
                   const std::string& module, 
                   int                line=0, 
                   const std::string& file="") const throw (AssertionError);

   template <class T>
   void compareNot(const T&           t1, 
                   const T&           t2,
                   const std::string& module, 
                   int                line=0, 
                   const std::string& file="") const throw (AssertionError)
   {
      if (t1==t2)
      {
         std::ostringstream reason;
         reason << t1 << "!=" << t2;
         throw AssertionError("AssertionError", reason.str(), 
                              module, line, file);
      }
   }

protected:
   TestDriver(const std::string& driverName);

   virtual void preExecute() {}
   virtual void execute() = 0;
   virtual void postExecute() {}

   virtual std::ostream& logInfo  (uint line=0, const std::string& file="");
   virtual std::ostream& logResult(uint line=0, const std::string& file="");
   virtual std::ostream& logError (uint line=0, const std::string& file="");

   void testing(const std::string& module, 
                unsigned int       line=0, 
                const std::string& file="");

private:
   // Illegal
   TestDriver(const TestDriver&);
   TestDriver& operator=(const TestDriver&);

   std::ostream& logStream(std::ostream& os, uint line, const std::string& file);

   std::string  theDriverName;
   TestHarness* theTestHarness;
};


} // namespace SoftTrac


#endif
