//-----------------------------------------------------------------------------
// FILE:              TestHarness.h
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_TestHarness_H
#define SoftTrac_TestHarness_H

#include <iostream>
#include <list>
#include <map>
#include <string>

namespace SoftTrac {

class TestDriver;

//-----------------------------------------------------------------------------
// CLASS: TestHarness
//-----------------------------------------------------------------------------
class TestHarness
{
public:
   enum { MAX_SECS_PER_DRIVER = 60 };

   TestHarness(int           argc, 
               char**        argv,
               std::ostream& infoStream   =std::cout,
               std::ostream& resultsStream=std::clog,
               std::ostream& errorStream  =std::cerr,
               uint          abortAfter   =MAX_SECS_PER_DRIVER);

   const std::string& programName() const { return theProgramName; }

   int         argc() const       { return theArgc; }
   std::string argv(uint i) const { return theArgv[i]; }

   std::ostream& infoStream()    { return theInfoStream;    }
   std::ostream& resultsStream() { return theResultsStream; }
   std::ostream& errorStream()   { return theErrorStream;   }

   static void registerDriver(TestDriver*);

private:
   //--------------------------------------------------------------------------
   // CLASS: WatchDog
   //--------------------------------------------------------------------------
   class WatchDog
   {
   public:
      WatchDog(uint seconds);
     ~WatchDog();
   private:
      WatchDog();
   };

   void testDrivers(const std::list<TestDriver*>&);

   std::ostream& header(TestDriver*);
   std::ostream& footer();

   std::string   theProgramName;
   int           theArgc;
   char**        theArgv;
   std::ostream& theInfoStream;
   std::ostream& theResultsStream;
   std::ostream& theErrorStream;
   int           theMaxSecsPerDriver;

   typedef std::map<std::string, TestDriver*, std::less<std::string> > Registry;

   static Registry* registry;
};


} // namespace SoftTrac

#endif
