//-----------------------------------------------------------------------------
// FILE:              Trace.h
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_Trace_h
#define SoftTrac_Trace_h

#include <iostream>
#include <string>


namespace SoftTrac {

//-----------------------------------------------------------------------------
// CLASS: Trace
//-----------------------------------------------------------------------------
class Trace 
{
public:
   enum Flag { ON, OFF };

   Trace(const std::string& className, Flag flag=OFF):theClassName(className)
   { if (flag == ON) setAll(); }

   bool enter() const { return theEnterFlag; }
   bool exit()  const { return theExitFlag;  }
   bool debug() const { return theDebugFlag; }
   bool log()   const { return theLogFlag;   }
   bool error() const { return theErrorFlag; }
   bool user()  const { return theUserFlag;  }

   void enter(const std::string& method) 
   { if (theEnterFlag) output(std::cout, "Enter", method, ""); }
   
   void exit (const std::string& method) 
   { if (theExitFlag)  output(std::cout, "Exit", method, ""); }
   
   void debug(const std::string& method, const std::string& message) 
   { if (theDebugFlag) output(std::cerr, "Debug", method, message); }
   
   void log  (const std::string& method, const std::string& message) 
   { if (theLogFlag) output(std::cout, "Log", method, message); }
   
   void error(const std::string& method, const std::string& message) 
   { if (theErrorFlag) output(std::cerr, "Error", method, message); }
   
   void user (const std::string& method, const std::string& message) 
   { if (theUserFlag) output(std::cout, "User", method, message); }

   void setAll();
   void setEnter() { theEnterFlag = true; }
   void setExit()  { theExitFlag  = true; }
   void setDebug() { theDebugFlag = true; }
   void setLog()   { theLogFlag   = true; }
   void setError() { theErrorFlag = true; }
   void setUser()  { theUserFlag  = true; }

   void clearAll();
   void clearEnter() { theEnterFlag = false; }
   void clearExit()  { theExitFlag  = false; }
   void clearDebug() { theDebugFlag = false; }
   void clearLog()   { theLogFlag   = false; }
   void clearError() { theErrorFlag = false; }
   void clearUser()  { theUserFlag  = false; }

   void output(std::ostream&      out,
               const std::string& type,
               const std::string& method,
               const std::string& message);

   std::string theClassName;

   bool theEnterFlag;
   bool theExitFlag;
   bool theDebugFlag;
   bool theLogFlag;   
   bool theErrorFlag;
   bool theUserFlag;
};

} // namespace SoftTrac


#endif