//---------------------------------------------------------------------------
// NAME: Timothy R. Culp
// FILE: commonExceptions.cpp
//---------------------------------------------------------------------------

#include "tconfig.h"
#include <sstream>
#include "./commonExceptions.h"

namespace SoftTrac {

const char* NotImplemented::WHAT = "Not implemented yet";


//---------------------------------------------------------------------------
// FormatError::FormatError
//---------------------------------------------------------------------------
FormatError::FormatError(const std::string& what,
	                      const std::string& who,
	                      int                line,
	                      const std::string& file) throw()
   :
      RuntimeError       ("FormatError", what, who, line, file),
      std::runtime_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// FormatError::FormatError
//---------------------------------------------------------------------------
FormatError::FormatError(const std::string& name,
                         const std::string& what,
	                      const std::string& who,
	                      int                line,
	                      const std::string& file) throw()
   :
      RuntimeError       (name, what, who, line, file),
      std::runtime_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// FileError::FileError
//---------------------------------------------------------------------------
FileError::FileError(const std::string& what,
	                  const std::string& filename,
	                  const std::string& who,
	                  int                line,
	                  const std::string& file) throw()
   :
      RuntimeError       ("FileError", what+ " - " + filename, who, line, file),
      std::runtime_error (what.c_str()),
      theFilename        (filename)
{
}


//---------------------------------------------------------------------------
// FileError::FileError
//---------------------------------------------------------------------------
FileError::FileError(const std::string& name,
                     const std::string& what,
	                  const std::string& filename,
	                  const std::string& who,
	                  int                line,
	                  const std::string& file) throw()
   :
      RuntimeError       (name, what+ " - " + filename, who, line, file),
      std::runtime_error (what.c_str()),
      theFilename        (filename)
{
}


//---------------------------------------------------------------------------
// NotImplemented::NotImplemented
//---------------------------------------------------------------------------
NotImplemented::NotImplemented(const std::string& who,
	                            int                line,
	                            const std::string& file) throw()
   :
      LogicError       ("NotImplemented", WHAT, who, line, file),
      std::logic_error (WHAT)

{
}


//---------------------------------------------------------------------------
// Usage::Usage
//---------------------------------------------------------------------------
Usage::Usage(int                argc,
             char**             argv,
             int                expArgc,
             const std::string& expArgv,
             const std::string& who,
             int                line,
             const std::string& file) throw()
   :
      InvalidArgument("Usage", "", who, line, file)
{
   std::ostringstream what;
   what << "Only " << argc << " argument(s) when " << expArgc << " expected.\n";
   what << argv[0] << ' ' << expArgv << '\n';
   setWhat(what.str());
}


} // namespace SoftTrac
