//-----------------------------------------------------------------------------
// FILE:              CommonExceptions.h
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_CommonExceptions_H
#define SoftTrac_CommonExceptions_H

#include "./Exception.h"
#include "util/strings/Filename.h"
#include "util/util_api.h"

namespace SoftTrac {


//---------------------------------------------------------------------------
// CLASS: FormatError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API FormatError : public RuntimeError,
                        public std::runtime_error
{
public:
   FormatError(const std::string& what,
	            const std::string& who,
	            int                line=0,
	            const std::string& file="") throw();

protected:
   FormatError(const std::string& name,
               const std::string& what,
	            const std::string& who,
	            int                line=0,
	            const std::string& file="") throw();
};


//---------------------------------------------------------------------------
// CLASS: FileError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API FileError : public RuntimeError,
                                    public std::runtime_error
{
public:
   FileError(const std::string& what,
             const std::string& filename,
             const std::string& who,
             int                line=0,
             const std::string& file="") throw();

   Filename filename() const { return theFilename; }

protected:
   FileError(const std::string& name,
             const std::string& what,
             const std::string& filename,
             const std::string& who,
             int                line=0,
             const std::string& file="") throw();

   std::string theFilename;
};


//---------------------------------------------------------------------------
// CLASS: NotImplemented
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API NotImplemented : public LogicError,
                                         public std::logic_error
{
public:
   NotImplemented(const std::string& who,
	               int                line=0,
	               const std::string& file="") throw();

private:
   static const char* WHAT;
};


//---------------------------------------------------------------------------
// CLASS: Usage
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API Usage : public InvalidArgument
{
public:
   Usage(int                argc,
         char**             argv,
         int                expArgc,
         const std::string& expArgv,
         const std::string& who,
         int                line=0,
         const std::string& file="") throw();
};


} // namespace SoftTrac;


#endif
