//-----------------------------------------------------------------------------
// FILE:              TestXyVector.cpp
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include "./TestXyVector.h"
#include "util/points/XyVector.h"

namespace SoftTrac {

TestXyVector TestXyVector::registerThis;


//-----------------------------------------------------------------------------
// TestXyVector::execute
//-----------------------------------------------------------------------------
void TestXyVector::execute()
TRY_EXCEPTION
{
   testEqual();
   testLessThan();
   logInfo(__LINE__, __FILE__) << name() << " PASSED." << std::endl;
}
RETHROW_EXCEPTION("TestXyVector::execute");


//-----------------------------------------------------------------------------
// TestXyVector::testEqual
//-----------------------------------------------------------------------------
void TestXyVector::testEqual()
TRY_EXCEPTION
{
   static const char* const MODULE = "TestXyVector::testEqual";

   testing(MODULE, __LINE__, __FILE__);

   XyVector<int> pt0(0,0);
   XyVector<int> pt1(0,1);
   XyVector<int> pt2(1,0);
   XyVector<int> pt3(1,1);

   compare   (pt0, pt0, MODULE, __LINE__, __FILE__);
   compareNot(pt0, pt1, MODULE, __LINE__, __FILE__);
   compareNot(pt0, pt2, MODULE, __LINE__, __FILE__);
   compareNot(pt0, pt3, MODULE, __LINE__, __FILE__);

   compareNot(pt1, pt0, MODULE, __LINE__, __FILE__);
   compare   (pt1, pt1, MODULE, __LINE__, __FILE__);
   compareNot(pt1, pt2, MODULE, __LINE__, __FILE__);
   compareNot(pt1, pt3, MODULE, __LINE__, __FILE__);

   compareNot(pt2, pt0, MODULE, __LINE__, __FILE__);
   compareNot(pt2, pt1, MODULE, __LINE__, __FILE__);
   compare   (pt2, pt2, MODULE, __LINE__, __FILE__);
   compareNot(pt2, pt3, MODULE, __LINE__, __FILE__);

   compareNot(pt3, pt0, MODULE, __LINE__, __FILE__);
   compareNot(pt3, pt1, MODULE, __LINE__, __FILE__);
   compareNot(pt3, pt2, MODULE, __LINE__, __FILE__);
   compare   (pt3, pt3, MODULE, __LINE__, __FILE__);
}
RETHROW_EXCEPTION("TestXyVector::testEqual");


//-----------------------------------------------------------------------------
// TestXyVector::testLessThan
//-----------------------------------------------------------------------------
void TestXyVector::testLessThan()
TRY_EXCEPTION
{
   static const char* const MODULE = "TestXyVector::testLessThan";

   testing(MODULE, __LINE__, __FILE__);

   XyVector<int> pt0(-1,-1);
   XyVector<int> pt1( 0,-1);
   XyVector<int> pt2( 1,-1);
   XyVector<int> pt3(-1, 0);
   XyVector<int> pt4( 0, 0);
   XyVector<int> pt5( 1, 0);
   XyVector<int> pt6(-1, 1);
   XyVector<int> pt7( 0, 1);
   XyVector<int> pt8( 1, 1);

   compareNot((pt0<pt0), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt1), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt2), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt3), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt4), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt5), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt6), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt0<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt1<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt1<pt1), MODULE, __LINE__, __FILE__);
   compare   ((pt1<pt2), MODULE, __LINE__, __FILE__);
   compare   ((pt1<pt3), MODULE, __LINE__, __FILE__);
   compare   ((pt1<pt4), MODULE, __LINE__, __FILE__);
   compare   ((pt1<pt5), MODULE, __LINE__, __FILE__);
   compare   ((pt1<pt6), MODULE, __LINE__, __FILE__);
   compare   ((pt1<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt1<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt2<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt2<pt1), MODULE, __LINE__, __FILE__);
   compareNot((pt2<pt2), MODULE, __LINE__, __FILE__);
   compare   ((pt2<pt3), MODULE, __LINE__, __FILE__);
   compare   ((pt2<pt4), MODULE, __LINE__, __FILE__);
   compare   ((pt2<pt5), MODULE, __LINE__, __FILE__);
   compare   ((pt2<pt6), MODULE, __LINE__, __FILE__);
   compare   ((pt2<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt2<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt3<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt3<pt1), MODULE, __LINE__, __FILE__);
   compareNot((pt3<pt2), MODULE, __LINE__, __FILE__);
   compareNot((pt3<pt3), MODULE, __LINE__, __FILE__);
   compare   ((pt3<pt4), MODULE, __LINE__, __FILE__);
   compare   ((pt3<pt5), MODULE, __LINE__, __FILE__);
   compare   ((pt3<pt6), MODULE, __LINE__, __FILE__);
   compare   ((pt3<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt3<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt4<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt4<pt1), MODULE, __LINE__, __FILE__);
   compareNot((pt4<pt2), MODULE, __LINE__, __FILE__);
   compareNot((pt4<pt3), MODULE, __LINE__, __FILE__);
   compareNot((pt4<pt4), MODULE, __LINE__, __FILE__);
   compare   ((pt4<pt5), MODULE, __LINE__, __FILE__);
   compare   ((pt4<pt6), MODULE, __LINE__, __FILE__);
   compare   ((pt4<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt4<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt5<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt5<pt1), MODULE, __LINE__, __FILE__);
   compareNot((pt5<pt2), MODULE, __LINE__, __FILE__);
   compareNot((pt5<pt3), MODULE, __LINE__, __FILE__);
   compareNot((pt5<pt4), MODULE, __LINE__, __FILE__);
   compareNot((pt5<pt5), MODULE, __LINE__, __FILE__);
   compare   ((pt5<pt6), MODULE, __LINE__, __FILE__);
   compare   ((pt5<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt5<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt6<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt6<pt1), MODULE, __LINE__, __FILE__);
   compareNot((pt6<pt2), MODULE, __LINE__, __FILE__);
   compareNot((pt6<pt3), MODULE, __LINE__, __FILE__);
   compareNot((pt6<pt4), MODULE, __LINE__, __FILE__);
   compareNot((pt6<pt5), MODULE, __LINE__, __FILE__);
   compareNot((pt6<pt6), MODULE, __LINE__, __FILE__);
   compare   ((pt6<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt6<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt7<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt7<pt1), MODULE, __LINE__, __FILE__);
   compareNot((pt7<pt2), MODULE, __LINE__, __FILE__);
   compareNot((pt7<pt3), MODULE, __LINE__, __FILE__);
   compareNot((pt7<pt4), MODULE, __LINE__, __FILE__);
   compareNot((pt7<pt5), MODULE, __LINE__, __FILE__);
   compareNot((pt7<pt6), MODULE, __LINE__, __FILE__);
   compareNot((pt7<pt7), MODULE, __LINE__, __FILE__);
   compare   ((pt7<pt8), MODULE, __LINE__, __FILE__);

   compareNot((pt8<pt0), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt1), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt2), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt3), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt4), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt5), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt6), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt7), MODULE, __LINE__, __FILE__);
   compareNot((pt8<pt8), MODULE, __LINE__, __FILE__);

   assert (XyVector<int> (1,0) < XyVector<int>  (0, 1));
}
RETHROW_EXCEPTION("TestXyVector::testLessThan");


} // namespace SoftTrac
