//-----------------------------------------------------------------------------
// FILE:              Filename.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include <fstream>
#include "./Filename.h"
#include "util/exceptions/CommonExceptions.h"

using namespace std;

namespace SoftTrac {


//---------------------------------------------------------------------------
// Filename::~Filename
//---------------------------------------------------------------------------
Filename::~Filename()
{
   delete theBaseName;
}


//---------------------------------------------------------------------------
// Filename::size
//---------------------------------------------------------------------------
uint Filename::size() const
{
   static const char* MODULE = "Filename::size";

   ifstream is(c_str());
   if (!is)
   {
      throw FileError("File open error", *this, MODULE, __LINE__, __FILE__);
   }

   is.seekg(0, ios::end);
   return is.tellg();
}


//---------------------------------------------------------------------------
// Filename::exists
//---------------------------------------------------------------------------
bool Filename::exists() const
{
   static const char* MODULE = "Filename::exists";

   ifstream is(c_str());
   if (!is)
   {
      throw FileError("File open error", *this, MODULE, __LINE__, __FILE__);
   }

   return is.good();
}


//---------------------------------------------------------------------------
// Filename::baseName
//---------------------------------------------------------------------------
const Filename& Filename::baseName() const
TRY_EXCEPTION
{
   if (empty()) return *this;

   if (!theBaseName)
   {
      int i = length() - 1;
      bool done = false;
      while (!done && i)
      {
         char c = at(i);
         done = (c == ':' || c == '/' || c == '\\');
         if (!done) --i; else ++i;
      }
 
      theBaseName = new Filename(substr(i));
   }

   return *theBaseName;
}
RETHROW_EXCEPTION("Filename::baseName");


} // namespace SoftTrac
