//-----------------------------------------------------------------------------
// FILE:  Substring.cpp
//-----------------------------------------------------------------------------

#include <iostream>
#include <string.h>
#include "tconfig.h"
#include "./SubString.h"

using namespace std;

namespace SoftTrac {


//-----------------------------------------------------------------------------
// Substring::Substring
//-----------------------------------------------------------------------------
Substring::Substring(string& str, string& str2, size_type pos)
   :
      theStr(&str),
      theLen(str2.length())
{
   thePos = str.find(str2, pos);
}


//-----------------------------------------------------------------------------
// Substring::Substring
//-----------------------------------------------------------------------------
Substring::Substring(string& str, char* c_str, size_type pos)
   :
      theStr(&str),
      theLen(strlen(c_str))
{
   thePos = (c_str) ? str.find(c_str, pos) : string::npos;
}


//-----------------------------------------------------------------------------
// Substring::operator=
//-----------------------------------------------------------------------------
const Substring& Substring::operator=(const string& str)
{
   if (thePos == string::npos) return *this;

   theStr->replace(thePos, theLen, str);
   theLen = str.length();

   return *this;
}


//-----------------------------------------------------------------------------
// Substring::operator=
//-----------------------------------------------------------------------------
const Substring& Substring::operator=(const Substring& substr)
{
   if (thePos == string::npos) return *this;

   theStr->replace(thePos,theLen,*substr.theStr,substr.thePos,substr.theLen);
   theLen = substr.theLen;

   return *this;
}


//-----------------------------------------------------------------------------
// Substring::operator=
//-----------------------------------------------------------------------------
const Substring& Substring::operator=(const char* c_str)
{
   if (thePos == string::npos) return *this;

   if (!c_str)
   {
      theStr->replace(thePos, theLen, c_str);
      theLen = strlen(c_str);
   }
   else
   {
      string& s = *theStr;
      theStr->erase(&s[thePos], &s[thePos] + theLen);
      theLen = 0;
   }

   return *this;
}


//-----------------------------------------------------------------------------
// Substring::operator=
//-----------------------------------------------------------------------------
const Substring& Substring::operator=(char ch)
{
   if (thePos == string::npos) return *this;

   theStr->replace(thePos, theLen, 1, ch);
   theLen = 1;

   return *this;
}


//-----------------------------------------------------------------------------
// Substring::string
//-----------------------------------------------------------------------------
Substring::operator std::string() const
{
   if (thePos == string::npos) return string("");
   return string(*theStr, thePos, theLen);
}


} // namespace SoftTrac


//-----------------------------------------------------------------------------
// operator<<
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const SoftTrac::Substring& substr)
{
   return os << string(substr);
}


