//-----------------------------------------------------------------------------
// FILE:  Substring.h
//-----------------------------------------------------------------------------
#ifndef Substring_H
#define Substring_H

#include <iosfwd>
#include <string>
#include "util/util_api.h"

namespace SoftTrac {

//-----------------------------------------------------------------------------
// CLASS:  Substring
//-----------------------------------------------------------------------------
class SOFTTRAC_UTIL_API Substring
{
public:
   typedef std::string::size_type size_type;

   Substring(std::string& str, size_type pos, size_type len);
   Substring(std::string& str, std::string& str2, size_type pos=0);
   Substring(std::string& str, char* c_str, size_type pos=0);

   const Substring& operator=(const std::string&);
   const Substring& operator=(const Substring&);
   const Substring& operator=(const char*);
   const Substring& operator=(char);

   operator std::string() const;

   bool isValid()    const { return thePos != std::string::npos; }
   bool isNotValid() const { return !isValid(); }
   bool operator!()  const { return !isValid(); }

   size_type position() const { return thePos; }
   size_type length()   const { return theLen; }

private:
   std::string* theStr;
   size_type    thePos;
   size_type    theLen;
};


//-----------------------------------------------------------------------------
// Substring::Substring
//-----------------------------------------------------------------------------
inline Substring::Substring(std::string& str, size_type pos, size_type len)
   : 
      theStr(&str), 
      thePos(pos<str.size() ? pos : std::string::npos), 
      theLen(len)
{
}


} // namespace SoftTrac


std::ostream& operator<<(std::ostream&, const SoftTrac::Substring&);


#endif

