//-----------------------------------------------------------------------------
// FILE:              TestExample.cpp
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include "./TestExample.h"

namespace SoftTrac {

TestExample TestExample::registerThis;


//-----------------------------------------------------------------------------
// TestExample::execute
//-----------------------------------------------------------------------------
void TestExample::execute()
TRY_EXCEPTION
{
   testCase1();
   testCase2();
   logInfo(__LINE__, __FILE__) << name() << " PASSED." << std::endl;
}
RETHROW_EXCEPTION("TestExample::execute");


//-----------------------------------------------------------------------------
// TestExample::testCase1
//
// This example shows how to test for true and false expressions.
//-----------------------------------------------------------------------------
void TestExample::testCase1()
TRY_EXCEPTION
{
   static const char* const MODULE = "TestExample::testCase1";

   testing(MODULE, __LINE__, __FILE__);

   bool trueExpr  = true;
   bool falseExpr = false;

   compare   (trueExpr,  MODULE, __LINE__, __FILE__);
   compareNot(falseExpr, MODULE, __LINE__, __FILE__);
}
RETHROW_EXCEPTION("TestExample::testCase1");


//-----------------------------------------------------------------------------
// TestExample::testCase2
//
// This example shows how to test for equality and inequality. The compare
// functions will work for any type T that supports operator== and operator<<.
//-----------------------------------------------------------------------------
void TestExample::testCase2()
TRY_EXCEPTION
{
   static const char* const MODULE = "TestExample::testCase2";

   testing(MODULE, __LINE__, __FILE__);

   compare   ( 2,   2,  MODULE, __LINE__, __FILE__); // int
   compare   ('2', '2', MODULE, __LINE__, __FILE__); // char
   compareNot( 2,   3,  MODULE, __LINE__, __FILE__); // int
   compareNot('2', '3', MODULE, __LINE__, __FILE__); // char
}
RETHROW_EXCEPTION("TestExample::testCase2");


} // namespace SoftTrac
