//-----------------------------------------------------------------------------
// NAME: Timothy R. Culp
// FILE: tfunctional.h
//-----------------------------------------------------------------------------
#ifndef tfunctionalH
#define tfunctionalH

#include <functional>
#include "util/util_api.h"

namespace SoftTrac {

//-----------------------------------------------------------------------------
// CLASS: BinaryPlusEqual
//-----------------------------------------------------------------------------
template <class T>
class BinaryPlusEqual : public std::binary_function<T, T, T>
{
public:
   T& operator()(T& t1, const T& t2) const { return t1 += t2; } 
};


//-----------------------------------------------------------------------------
// CLASS: BinaryMinusEqual
//-----------------------------------------------------------------------------
template <class T>
class BinaryMinusEqual : public std::binary_function<T, T, T>
{
public: 
   T& operator()(T& t1, const T& t2) const { return t1 -= t2; }
};


//-----------------------------------------------------------------------------
// CLASS: BinaryMultiplyEqual
//-----------------------------------------------------------------------------
template <class T>
class BinaryMultiplyEqual : public std::binary_function<T, T, T>
{
public:
   T& operator()(T& t1, const T& t2) const { return t1 *= t2; }
};


//-----------------------------------------------------------------------------
// CLASS: BinaryDivideEqual
//-----------------------------------------------------------------------------
template <class T>
class BinaryDivideEqual : public std::binary_function<T, T, T>
{
public:
   T& operator()(T& t1, const T& t2) const { return t1 /= t2; }
};


//-----------------------------------------------------------------------------
// CLASS: UnaryPlusEqual
//-----------------------------------------------------------------------------
template <class T>
class UnaryPlusEqual : public std::unary_function<T, T>
{
public:
   UnaryPlusEqual(const T& t) : theT(t) {}

   T& operator()(T& t) const { return t += theT; }

private:
   T theT;
};


//-----------------------------------------------------------------------------
// CLASS: UnaryMinusEqual
//-----------------------------------------------------------------------------
template <class T>
class UnaryMinusEqual : public std::unary_function<T, T>
{
public:
   UnaryMinusEqual(const T& t) : theT(t) {}

   T& operator()(T& t) const { return t -= theT; }

private:
   T theT;
};


//-----------------------------------------------------------------------------
// CLASS: UnaryMultiplyEqual
//-----------------------------------------------------------------------------
template <class T>
class UnaryMultiplyEqual : public std::unary_function<T, T>
{
public:
   UnaryMultiplyEqual(const T& t) : theT(t) {}

   T& operator()(T& t) const { return t *= theT; }

private:
   T theT;
};


//-----------------------------------------------------------------------------
// CLASS: UnaryDivideEqual
//-----------------------------------------------------------------------------
template <class T>
class UnaryDivideEqual : public std::unary_function<T, T>
{
public:
   UnaryDivideEqual(const T& t) : theT(t) {}

   T& operator()(T& t) const { return t /= theT; }

private:
	T theT;
};


//-----------------------------------------------------------------------------
// CLASS: destroy,destroyArray
//-----------------------------------------------------------------------------
template <class T>
class destroy : public std::unary_function<void, T>
{
public:
   void operator()(T* t) const { delete t; }
};

template <class T>
class destroyArray : public std::unary_function<void, T>
{
public:
   void operator()(T* t) const { delete[] t; }
};


//-----------------------------------------------------------------------------
// CLASS: EqualTo
//-----------------------------------------------------------------------------
template <class T>
class EqualTo : public std::unary_function<T, bool>
{
public:
   EqualTo(const T& t) : theT(t) {}
   bool operator()(T& t) const { return t == theT; }
private:
   T theT;
};


//-----------------------------------------------------------------------------
// CLASS: Rand
//-----------------------------------------------------------------------------
class SOFTTRAC_UTIL_API Rand : public std::unary_function<ulong, ulong>
{
public:
   Rand();
   ulong operator()(ulong n);
private:
   static bool isSeeded;
};

} // namespace SoftTrac


#endif
